#    Pandoc Flow - Augments Pandoc with advanced pre-processing, automates routine
#    Copyright (C) 2021 Dmitry Berezovsky
#    The MIT License (MIT)
#
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import strome.processors.jinja2
from strome.pipeline import StromeRuntime

__all__ = ("Jinja2Processor",)


class Jinja2Processor(strome.processors.jinja2.Jinja2Processor):
    def prepare_rendering_context(self, context: dict, flow_runtime: StromeRuntime):
        context.update(
            dict(
                pandoc_variable=flow_runtime.config.get("variables", {}),
                pandoc_metadata=flow_runtime.config.get("metadata", {}),
                pandoc_config=flow_runtime.config,
            )
        )



#    Pandoc Flow - Augments Pandoc with advanced pre-processing, automates routine
#    Copyright (C) 2021 Dmitry Berezovsky
#    The MIT License (MIT)
#
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
from copy import copy
from typing import Optional

import pandoc_flow.const
from strome import const
from strome.error import ProcessorLoadingError
from pandoc_flow.pipeline import PandocFlowRuntime
from strome.pipeline import PipelineElement, ExternalExecutableMixin
from strome.utils import dict_to_yaml_file

__all__ = ("PandocProcessor",)


class PandocProcessor(PipelineElement, ExternalExecutableMixin):
    NAME = "pandoc"

    def __init__(self) -> None:
        super().__init__()
        self.pandoc_version: Optional[str] = None
        self.pandoc_executable = "pandoc"

    def load(self):
        self.logger.debug("Validating pandoc installation")
        if not self.is_successful_exit_code(self.pandoc_executable, "--version"):
            raise ProcessorLoadingError(
                "Pandoc executable doesn't exist.",
                self,
                recommendation="Check if you have pandoc installed and available in PATH",
            )

    def setup(self, flow_runtime: PandocFlowRuntime, params: dict):
        self.logger.info("Configuring pandoc")
        pandoc_version_result = self.run_executable(self.pandoc_executable, "--version", hide_output=True)
        lines = pandoc_version_result.stdout.split("\n")
        if lines[0].startswith("pandoc"):
            self.pandoc_version = lines[0].replace("pandoc", "").strip()
            self.logger.info("\t\tDetected pandoc version: " + self.pandoc_version)  # type: ignore

    def __generate_pandoc_yaml_conf(self, flow_runtime: PandocFlowRuntime):
        # Generate and write pandoc yaml
        flow_runtime.pandoc_config_generated_path = os.path.join(
            flow_runtime.temp_dir, os.path.basename(flow_runtime.config_path)
        )
        generated_pandoc_conf = copy(flow_runtime.config)
        # Ensure we remove all config options not supported by pandoc
        del generated_pandoc_conf[pandoc_flow.const.CONF_PANDOC_FLOW]
        del generated_pandoc_conf[const.CONF_DEFAULTS]
        if const.CONF_OUTPUT in generated_pandoc_conf:
            del generated_pandoc_conf[const.CONF_OUTPUT]
        # Extend input files with generated ones
        if const.CONF_INPUT_FILES in generated_pandoc_conf:
            generated_pandoc_conf[const.CONF_INPUT_FILES] = (
                flow_runtime.context[const.RUNTIME_INPUT_FILES] + generated_pandoc_conf[const.CONF_INPUT_FILES]
            )
        # Extend resource path
        resource_path = ["."]
        if const.CONF_RESOURCE_PATH in generated_pandoc_conf:
            if isinstance(generated_pandoc_conf[const.CONF_RESOURCE_PATH], list):
                resource_path = generated_pandoc_conf[const.CONF_RESOURCE_PATH]
            elif isinstance(generated_pandoc_conf[const.CONF_RESOURCE_PATH], str):
                resource_path = generated_pandoc_conf[const.CONF_RESOURCE_PATH].split(":")
            else:
                raise ValueError("resource-path must be either list or colon delimited string")
            resource_path += flow_runtime.context[const.RUNTIME_RESOURCE_PATH]
        generated_pandoc_conf[const.CONF_RESOURCE_PATH] = resource_path
        # Write down generated pandoc conf
        dict_to_yaml_file(generated_pandoc_conf, flow_runtime.pandoc_config_generated_path)

    def run(self, flow_runtime: PandocFlowRuntime):
        self.__generate_pandoc_yaml_conf(flow_runtime)
        output_file_path = flow_runtime.config.get("output")
        pandoc_args = [
            self.pandoc_executable,
            "-o",
            output_file_path,
            "--defaults",
            flow_runtime.pandoc_config_generated_path,
        ]
        for meta_var in flow_runtime.context[pandoc_flow.const.RUNTIME_PANDOC_META]:
            pandoc_args += ["-M", meta_var]
        for var in flow_runtime.context[pandoc_flow.const.RUNTIME_PANDOC_VARS]:
            pandoc_args += ["-V", var]
        self.logger.debug("Will be running pandoc with the following params: ")
        self.logger.debug(str(pandoc_args))
        self.run_executable(*pandoc_args)



#    Pandoc Flow - Augments Pandoc with advanced pre-processing, automates routine
#    Copyright (C) 2021 Dmitry Berezovsky
#    The MIT License (MIT)
#    
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import codecs
import os
from os import path

from setuptools import setup, find_packages

src_dir = path.abspath(path.dirname(__file__))
root_dir = path.join(src_dir, '..')

# == Read version ==
version_override = os.environ.get('VERSION_OVERRIDE', None) or None


def read_file(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read_file(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


if not version_override:
    version = get_version("pandoc_flow/__version__.py")
else:
    print("Using overridden version: " + version_override)
    version = version_override


# == END: Read version ==

def read_requirements(file_name: str):
    with open(file_name, 'r') as f:
        result = f.readlines()
    return list(filter(lambda l: not (l.startswith('#') or l.strip() == ''), result))


# Get the long description from the README file
readme_file = path.join(root_dir, 'docs/pypi-description.md')
try:
    from m2r import parse_from_file

    long_description = parse_from_file(readme_file)
except ImportError:
    # m2r may not be installed in user environment
    with open(readme_file) as f:
        long_description = f.read()

# Requirements
requirements_file = path.join(root_dir, 'requirements.txt')
requirements = read_requirements(requirements_file)

setup(
    name='pandoc-flow',
    # Semantic versioning should be used:
    # https://packaging.python.org/distributing/?highlight=entry_points#semantic-versioning-preferred
    version=version,
    description='Pandoc Flow - Augments Pandoc with advanced pre-processing, automates routine',
    long_description=long_description,
    url='https://github.com/corvis/pandoc-flow',
    keywords='pandoc pre-process processing pipiline document generator',

    # Author
    author='Dmitry Berezovsky',

    # License
    license='MIT',

    # Technical meta
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Home Automation',

        # License (should match "license" above)
        'License :: OSI Approved :: MIT License',
        # Python versions support
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    python_requires='>3.6',

    # Structure
    packages=find_packages(include=['*'], where=src_dir),
    # py_modules=["app", 'cli', 'daemonize'],

    install_requires=requirements,

    # Extra dependencies might be installed with:
    # pip install -e .[dev,test]
    extras_require={},

    # test_suite='nose2.collector.collector',
    # tests_require=[
    #     'nose2==0.8.0',
    # ],

)
